FUNCTION gridlo,a,l,RANGE=range,SIGMA=sigma

; version 3.1
;
; 31 Jan 02 change so sigma set based on new grid, not old 
; 24 Jan 02 tidy up code and bugs
; 11 Dec 01 simplify Russ's modifications
; 31 Oct 01 Russ to speed it up with where commands
;  1 Oct 01 make counters long
; 27 Mar 97
; 29 May 93 created
;
;  a = input spectrum, need at least 2 columns, wavelength and flux
;  l = new wavelength grid (input)
;
;  FUNCTION output:  flux on new wavlength grid (output)
;
;  this procedure converts spectra from one wavelength grid to another
;  convolves a gaussian centered over each new
;    pixel to weight data from old spectrum
;  old warning message:
;    IF YOU'RE GOING FROM LOW to HIGH RES, use REGRID (or gridhi) instead!
;    this was because the old lo-res grid would leave data gaps in the new
;    with the Dec 01 - Jan 02improvements, there should be no gaps
;  outstanding issue, what to use for sigma if not declared
;    changed 31 Jan 02 to set sigma based on new wavelength grid

; setup

al=reform(a(0,*))
af=reform(a(1,*))
nal=n_elements(al)
alstop=nal-1
lstop=n_elements(l)-1
b=fltarr(lstop+1)
if (n_elements(range) eq 0) then range=0.0
if (n_elements(sigma) eq 0) then sigma=0.0

; reverse order if al and af if al is not a positive grid
; ditto for l, the new wavelength grid

if (al(0) gt al(alstop)) then begin
  al=rotate(al,2)
  af=rotate(af,2)
endif
if (l(0) gt l(lstop)) then l=rotate(l,2)

; set grid of deltas for old wavelength grid (dal) and new grid (del)

dal    = al-shift(al,1) 
dal(0) = al(1)-al(0)

del    = l-shift(l,1)
del(0) = l(1)-l(0)

; iterate through wavelength elements of new grid
; old and new wavelength grids are now positive

for i=long(0),lstop do begin
  sum=0.0
  wtsum=0.0

; the algorithm is
;   (1) find the position in the old grid corresponding to the new position
;   (2) set sigma and integration range about this point
;   (3) perform the weighted gaussian summation about this point
;
; (1) find alspot (set to 0 if new grid point too low)

  lam = l(i)
  alspot=max(where(lam gt al)) > 0

; (2a) set sigma and range if not already set (i.e. = 0.0)
; if sigma = 0.0, set so FWHM = wavelength spacing in the old grid
; if range = 0.0, set it to equal 3 sigma

;  if (sigma eq 0.0) then sigma = 0.424 * dal(alspot)
  if (sigma eq 0.0) then sigma = 0.424 * del(i)
  if (range eq 0.0) then range = 3*sigma

; (2b) find range of old grid counters within range of alspot
; alrange gives the indices over which to integrate

  alrange =  where(al ge lam-range and al le lam+range,alcount)
  minrange=min(alrange)
  maxrange=max(alrange)

; if alcount gives 0 pixels in range, pick alspot 
; and neighbors if alspot not at the end of the range

  if (alcount eq 0) then begin
    alrange = alspot
    alcount = 1
    if (alspot lt alstop) then begin
      alrange = [alrange, alspot+1]
      alcount = alcount+1
    endif 
    if (alspot gt 0) then begin
      alrange = [alspot-1, alrange]
      alcount = alcount+1
    endif
  endif

; add one data point to each end of range, unless at 0 or last point
; this would guarantee at least 3 data points at ends, and 5 in middle

  if (alcount eq 0) then print,'error in gridlo at ',i ; this shouldn't happen
  if (minrange gt 0 and maxrange lt alstop) then begin
    alrange = [minrange-1,alrange,maxrange+1]
    alcount = alcount+2
  endif else begin
    if (minrange eq 0) then begin
      alrange = [alrange,maxrange+1]
      alcount = alcount+1
    endif
    if (maxrange eq alstop) then begin
      alrange = [minrange-1,alrange]
      alcount = alcount+1
    endif
  endelse

; (3) integrate:  gaussian weighted sum over range

  if (alcount gt 0) then begin
    wt = exp(-((al(alrange)-lam)*(al(alrange)-lam))/(2.0*sigma*sigma))
    sum=total(wt*af(alrange))
    wtsum=total(wt)
 
;   load flux array

    if (wtsum gt 0.0) then b(i) = sum/wtsum else b(i) = 0
  endif else b(i) = 0

endfor

RETURN,b

end
