PRO seg2segin,fixed,fseg,adjust,aseg,ll,flag,nfa

; version 4.2 
;  8 May 02  to avoid call to splice_in2 when not needed
; 30 Apr 02  to fix slight output error (as to which seg normalized to which)
;  4 Apr 02  to pass error vectors to splice for new error calculation (v 4.1)
; 29 Mar 02  to accept normalization errors from splice_in in flux units
; 19 Mar 02  repaired output to log file for call to splice_in2.pro
;  7 Mar 02  to include propagated error from previous segments
;  5 Mar 02  used splice output to determine new errors
; 27 Feb 02  created by separating repeated loop from swsmake2.pro (v 3.0)
;
; seg2segin calls splice_in.pro or splice_in2.pro to normalize
; one segment to another using the region of overlap
;
; input
;   fixed    - fixed set of data
;   fseg     - segment number of fixed data
;   adjust   - adjustable set of data
;   aseg     - segment number of adjustable data
;   ll       - array of wavelengths giving the usable overlap range 
;   flag     - passed to splice routine as mode 
;              (0 for multiplicative, 1 for additive)
;
; output
;   adjust   - flux portion renormalized
;              error portion renormalized and adjusted
;   nfa      - two-parameter vector:  normalization, error in normalization,
;              (error is a ratio for flag=0, in flux units for flag=1)

; call splice_in or splice_in2, depending on the number of wavelength elements
; the photometric errors are passed for the fixed segment
; the spectroscopic errors are passed for the adjusted segment
; this propagates the normalization errors through the segments one by one
; the splice routines return nfa, the normalization factor and error

; if 4 elements and central 2 the same, remove them to avoid call to splice_in2

if (n_elements(ll) eq 4) then begin
  if (ll(1) eq ll(2)) then ll=[ll(0),ll(3)]
end 

case (n_elements(ll)) of
  2 : begin
    nfa=splice_in(fixed(0,0,*),fixed(1,0,*),fixed(3,0,*),$
      adjust(0,0,*),adjust(1,0,*),adjust(2,0,*),ll(0),ll(1),mode=flag)
    printf,99,format='("  splice_in  segment ",i2," to segment ",i2," at   ",f6.3," - ",f6.3," um, mode ",i1)',$
      aseg,fseg,ll(0),ll(1),flag
  end
  4 : begin
    nfa=splice_in2(fixed(0,0,*),fixed(1,0,*),fixed(3,0,*),adjust(0,0,*),$
      adjust(1,0,*),adjust(2,0,*),ll(0),ll(1),ll(2),ll(3),mode=flag)
    printf,99,format='("  splice_in2 segment ",i2," to segment ",i2," at   ",f6.3," - ",f6.3," um, mode ",i1)',$
      aseg,fseg,ll(0),ll(3),flag
    printf,99,format='("                              skipping     ",f6.3," - ",f6.3," um")',ll(1),ll(2)
  end
  else : begin
    printf,99,'seg2segin not normalizing, wrong number of lambda inputs'
    print,'seg2segin giving defaults, wrong number of lambda inputs'
    if (flag eq 0) then nfa=[1.0,0.0] else nfa=[0.0,0.0]
    end
endcase

; normalize the data in the adjustable segment
; adjust(1,0,*), flux multiplied or added to normalization factor
; adjust(2,0,*), spec. error, adjusted only for multiplicative case
;   do not re-normalize for additive shift
; adjust(3,0,*), photometric error
;   for flag=0 (multiplicative), use equation
;   e_ph^2 = e_sp^2 * nfa(0)^2 + old_flux^2 * nfa(1)^2
;   for flag=1 (additive), just add in quadrature since nfa(1) in flux units

if (flag eq 0) then begin
  old_flux      = adjust(1,0,*)
  adjust(1,0,*) = adjust(1,0,*) * nfa(0)                        ; flux
  adjust(2,0,*) = adjust(2,0,*) * nfa(0)                        ; spec. error
  adjust(3,0,*) = sqrt( adjust(2,0,*)^2 + old_flux^2*nfa(1)^2 ) ; ph. error
endif else begin
  adjust(1,0,*) = adjust(1,0,*) + nfa(0)                        ; flux
  adjust(3,0,*) = sqrt( adjust(2,0,*)^2 + nfa(1)^2 )            ; ph. error
endelse

return
end
