PRO seg2segout,fixed,fseg,adjust,aseg,type,flag,prop_err,nfa

; version 4.2
; 16 Jun 02  to modify wavelengths for Band 3E/4 interface (v 4.2)
;  5 Apr 02  to return to multiplicative error as a ratio (v 4.1)
; 29 Mar 02  to accept norm. errors from splice_out in flux units (v 4.0)
; 14 Mar 02  created by modifiying seg2segin.pro to call splice_out.pro (v 3.0)
;
; seg2segout calls splice_out.pro to use extrapolation to fit one
; one segment to another
;
; input
;   fixed    - fixed set of data
;   fseg     - segment number of fixed data
;   adjust   - adjustable set of data
;   aseg     - segment number of adjustable data
;   type     - sets wavelength ranges and degrees of polynomials 
;              for extrapolation
;   flag     - passed to splice routine as mode 
;              (0 for multiplicative, 1 for additive)
;   prop_err - propagated error from previous calls to seg2segin (in Jy)
;
; output
;   adjust   - flux portion renormalized
;              error portion renormalized and adjusted
;   nfa      - three-parameter vector:  normalization, error in normalization,
;              updated propagated error
;              (all in flux units, except first term for mult. correction)
; seg2segout will only function for calls to normalize segments 12 or 13

case aseg of
  13: begin
    print,format='("normalization of Band 4 (Seg. 13) of type ",i1)',type
    case type of
      0: begin                                                      ; standard
         la0=20.0 & la1=26.7 & lb0=27.7 & lb1=34.0 & lc=27.2 & dg=2
      end
      1: begin                                                      ; 26 um
         la0=26.0 & la1=27.3 & lb0=27.7 & lb1=34.0 & lc=27.5 & dg=1
      end
      2: begin                                                      ; bright red
         la0=23.5 & la1=27.3 & lb0=27.7 & lb1=31.0 & lc=27.5 & dg=1
      end
      3: begin                                                      ; noisy red
         la0=25.0 & la1=27.0 & lb0=27.7 & lb1=31.0 & lc=27.5 & dg=0
      end
    endcase
  end
  12: begin
    la0=27.7 & la1=31.0 & lb0=27.3 & lb1=27.7 & lc=27.5 & dg=0       ; Band 4/3E
  end
endcase

nfa=splice_out(fixed(0,0,*),fixed(1,0,*),la0,la1,$
      adjust(0,0,*),adjust(1,0,*),lb0,lb1,lc,degree=dg,mode=flag)

printf,99,format='("  splice_out segment ",i2," to segment ",i2,", type ",i1)',aseg,fseg,type 
printf,99,format='("        	",f6.3,"-",f6.3," ",f6.3,"-",f6.3," at ",f6.3," um, degree ",i1," mode ",i1)',$
  lb0,lb1,la0,la1,lc,dg,flag

if (flag eq 0) then begin
  old_flux      = adjust(1,0,*)
  adjust(1,0,*) = adjust(1,0,*) * nfa(0)                         ; flux
  adjust(2,0,*) = adjust(2,0,*) * nfa(0)                         ; spec. error
  adjust(3,0,*) = sqrt ( adjust(2,0,*)^2 + old_flux^2*nfa(1)^2 ) ; ph. error
;  adjust(3,0,*) = sqrt (adjust(3,0,*)^2 + prop_err^2)
  seg_avg = total(adjust(1,0,*))/float(n_elements(adjust(1,0,*)))
  prop_err = sqrt (prop_err^2 + nfa(1)^2*seg_avg^2)
endif else begin
  adjust(1,0,*) = adjust(1,0,*) + nfa(0)                         ; flux
  ; do not re-normalize spec. errors for additive shift
  adjust(3,0,*) = sqrt ( adjust(2,0,*)^2 + nfa(1)^2 )            ; ph. error
;  adjust(3,0,*) = sqrt (adjust(3,0,*)^2 + prop_err^2)
  prop_err = sqrt (prop_err^2 + nfa(1)^2)
endelse

return
end
