PRO seg2segtwo,fixed1,fseg1,adjust,aseg,fixed2,fseg2,flag,prop_err,nfa

; version 4.3
; 18 Dec 02  changes to comments only
; 22 Jul 02  created by modifying seg2segout.pro (v 4.3)
;
; seg2segtwo calls splice_two.pro to use extrapolation to fit one
; one segment to its two neighbors
;
; input
;   fixed1   - fixed set of data
;   fseg1    - segment number of fixed data
;   adjust   - adjustable set of data
;   aseg     - segment number of adjustable data
;   fixed2   - fixed set of data
;   fseg2    - segment number of fixed data
;   flag     - passed to splice routine as mode 
;              (0 for multiplicative, 1 for additive)
;   prop_err - propagated error from previous calls to seg2segin (in Jy)
;
; output
;   adjust   - flux portion renormalized
;              error portion renormalized and adjusted
;   nfa      - three-parameter vector:  normalization, error in normalization,
;              updated propagated error
;              (all in flux units, except first term for mult. correction)
; seg2segout will only function for calls to normalize segments 12 or 13

case aseg of
  12: begin
    la0=26.3 & la1=27.3 & lb0=27.3 & lb1=27.7 & lc0=27.7 & lc1=28.7
  end
endcase

nfa=splice_two(fixed1(0,0,*),fixed1(1,0,*),la0,la1,$
      adjust(0,0,*),adjust(1,0,*),lb0,lb1,$
      fixed2(0,0,*),fixed2(1,0,*),lc0,lc1,mode=flag)

printf,99,format=$
  '("  splice_two segment ",i2," to segments ",i2," and ",i2)',$
  aseg,fseg1,fseg2
printf,99,format=$
  '("        	",f6.3,"-",f6.3," ",f6.3,"-",f6.3," ",f6.3," - ",f6.3," um, mode ",i1)',$
  lb0,lb1,la0,la1,lc0,lc1,flag

if (flag eq 0) then begin
  old_flux      = adjust(1,0,*)
  adjust(1,0,*) = adjust(1,0,*) * nfa(0)                         ; flux
  adjust(2,0,*) = adjust(2,0,*) * nfa(0)                         ; spec. error
  adjust(3,0,*) = sqrt ( adjust(2,0,*)^2 + old_flux^2*nfa(1)^2 ) ; ph. error
;  adjust(3,0,*) = sqrt (adjust(3,0,*)^2 + prop_err^2)
  seg_avg = total(adjust(1,0,*))/float(n_elements(adjust(1,0,*)))
  prop_err = sqrt (prop_err^2 + nfa(1)^2*seg_avg^2)
endif else begin
  adjust(1,0,*) = adjust(1,0,*) + nfa(0)                         ; flux
  ; do not re-normalize spec. errors for additive shift
  adjust(3,0,*) = sqrt ( adjust(2,0,*)^2 + nfa(1)^2 )            ; ph. error
;  adjust(3,0,*) = sqrt (adjust(3,0,*)^2 + prop_err^2)
  prop_err = sqrt (prop_err^2 + nfa(1)^2)
endelse

return
end
