PRO segment,file_n,file_p,l_hi,final,err_hi,flag,SEG=seg,WT_N=wt_n

; version 4.1
; 16 Dec 02  changes to comments only
; 26 Apr 02  to handle a segment with no data (v 4.1)
; 25 Apr 02  to change sigpass from 3.0 to 4.0
; 20 Mar 02  to choose lo-res grid from larger of grids from 2 scan dir'ns
; 27 Feb 02  to pass segmenterr info needed to regrid errors itself
; 25 Jan 02  pass seg to segmenthigh
;  7 Aug 01  to write to log file ('all.lg1')
;  3 Aug 01  calls to new segmentmrg.pro and segmenterr.pro
; 31 Jul 01  to pass seg and wt_n, modify report output
; 20 Jul 01  add flag variable
; 23 Apr 01  add check for negatives in median
;            use segmentmad (instead of ...mod) in this case
; 29 Dec 00  add new routine to generate lo-res grid
; 14 Dec 00  to allow for cnt_n or cnt_p < 12
;  8 Nov 00
;
; see segmenttest.pro for previous version
; segmentall.pro before that
;
; master program to load and fully calibrate one spectral segment
;   makes all needed calls
;
; input:   file - name of input data
;                 must be processed using flslws, isotrim, isotrim
; output:  final - final fully calibrated spectrum

; set up initial parameters

npass=2
range=3
threshold=1.0
ndegree=3
sigpass=4.0
mpass=0.2

; obtain user-provided parameters or set them to default

if (n_elements(wt_n) eq 0) then wt_n=0.5    ; default = 0.5 
wt_p = 1.0 - wt_n                           ; should be 0.5
                                          
if (n_elements(seg) eq 0) then seg=0  ; only for bookkeeping

printf,99,''
printf,99,'Segment ',seg,' beginning'

; read input data for each scan direction

status_n=segmentread(file_n,raw_n,cnt_n,l_lo_n)
status_p=segmentread(file_p,raw_p,cnt_p,l_lo_p)

status=min(status_n,status_p)
if (status gt 0) then begin

; generate lo-res wavelength grid from larger of l_lo_n or l_lo_p

  if (n_elements(l_lo_n) ge n_elements(l_lo_p)) then begin
    l_lo = l_lo_n
  endif else begin
    l_lo = l_lo_p
  endelse

  printf,99,$
    format='("  lo-res grid ",f7.4," - ",f7.4,"  delta = ",f7.5," um")',$
    min(l_lo),max(l_lo),l_lo(1)-l_lo(0)

; generate median spectrum for each scan direction
;   both scan directions get the same wavelength grid
;   so all regridded data and median spectra are on the same grid

  segmentmed,raw_n,cnt_n,l_lo,grid_n,median_n,flag_n,npass=npass,range=range,threshold=threshold
  segmentmed,raw_p,cnt_p,l_lo,grid_p,median_p,flag_p,npass=npass,range=range,threshold=threshold

; generate new median from weights and "+" and "-" media
;   the two median vectors obviously must be on the same wavelength grid
;   also update the general flag variable

  segmentmrg,median_n,median_p,flag_n,flag_p,wt_n,median,flag,mpass=mpass

; if flag = 1, then median in one scan is too faint for standard treatment
;   if so, call segmentmad instead of segmentmod 
; correct data using polynomial fits so that the spectrum from each
;   detector matches the median spectrum
; segmentmod uses multiplicative corrections
; segmentmad uses additive corrections

  if (flag eq 0) then begin
    segmentmod,raw_n,cnt_n,l_lo,grid_n,median,mod_n,ndegree=ndegree
    segmentmod,raw_p,cnt_p,l_lo,grid_p,median,mod_p,ndegree=ndegree
  endif else begin
    segmentmad,raw_n,cnt_n,l_lo,grid_n,median,mod_n,ndegree=1
    segmentmad,raw_p,cnt_p,l_lo,grid_p,median,mod_p,ndegree=1
  endelse

; generate pass arrays, which define ranges of good data 
;   by compare the modified spectra from each detector to the median

  segmentpass,mod_n,cnt_n,l_lo,median,pass_n
  segmentpass,mod_p,cnt_p,l_lo,median,pass_p

; generate new pass by taking minimum of each element for "+" and "-" dir'ns
; this will eliminate spikes which appear in all detectors in only 1 dir'n
; modify by multiplying by sigpass

  len=max(size(l_lo)) ; OK since l_lo is one-dimensional
  pass=pass_n
  for i=0,len-1 do if (pass_p(i) lt pass_n(i)) then pass(i) = pass_p(i)
  pass = sigpass*pass

; merge count and moddata arrays for a single call to segmentsort

  nlen=max(size(cnt_n))
  plen=max(size(cnt_p))
  count=fltarr(nlen+plen)
  count(0:nlen-1)=cnt_n
  count(nlen:nlen+plen-1)=cnt_p
  len=max(count)
  len_n=max(cnt_n)
  len_p=max(cnt_p)
  moddata=fltarr(2,nlen+plen,len)
  moddata(*,0:nlen-1,0:len_n-1) = mod_n(*,*,0:len_n-1)
  moddata(*,nlen:nlen+plen-1,0:len_p-1) = mod_p(*,*,0:len_p-1)

; filter data using pass array and sort all detectors into wavelength order
;   use l_lo, the weighted median, and the minimum pass vector

  segmentsort,moddata,count,l_lo,median,pass,sortdata

; generate high-res wavelength array and regrid data to it

  segmenthigh,sortdata,seg,l_hi,final

; generate errors in high-res grid

  segmenterr,grid_n,grid_p,cnt_n,cnt_p,l_lo,l_hi,err_hi

  print,format='("Segment ",i2," complete, flag = ",i1,", down scan weight = ",f4.2)', seg,flag,wt_n
  printf,99,format='("Segment ",i2," complete, flag = ",i1,", down scan weight = ",f4.2)', seg,flag,wt_n

endif else begin
  l_hi=[0,0]
  final=[0,0]
  err_hi=[0,0]
  flag=1
endelse

end
