PRO segmenthigh,sortdata,seg,l_hi,finaldata

; version 3.0
; 27 Feb 02 cosmetic changes
; 31 Jan 02 fixed bug on lcnt line, changed lfreq(13) to 100
; 28 Jan 02 modified delta lambdas, new grid expands to cover actual range
; 25 Jan 02 recoded KK's changes
; 24 Jan 02 modified by KK to generate standard grid, based on speed and seg
;  5 Oct 00
;
; uses gridlo to shift spectrum to new hi-res wavelength grid
; input:   sortdata - sorted good data in one long 2-col array
;          seg      - segment number
; output:  l_hi = hi-res wavelength grid
;          finaldata - fluxes on hi-res wavelength grid

; step ONE - examine input data

sz = size(sortdata)
len = sz(2)
l_0 = min(sortdata(0,*))       ; just in case sortdata isn't sorted
l_1 = max(sortdata(0,*))       ; ditto
l_del = (l_1-l_0)/float(len-1) ; will use l_del to determine speed

; step TWO - create new wavelength grid
; based on seg and speed, as determined by l_del
;   lfreq  = no. of elements per um in new grid for SPEED 1
;   dtest = maximum l_del for SPEED 1 (in nm)

dltest = [0,0.34,0.45,0.45,0.70,0.70,1.5,1.4,0,2.4,2.4,5.0,5.2,6.2]
lfreq  = [0,2000,1500,1500,1000,1000,500,500,0,300,300,150,150,100]
lf     = lfreq(seg)

case 1 of
  (l_del gt dltest(seg)/1000): l_del=1.0/lf       ; speed 1
  (l_del lt dltest(seg)/2000): l_del=1.0/(4.0*lf) ; speed 4
  else                       : l_del=1.0/(2.0*lf) ; speed 3
endcase

; need to translate l_0 and l_1 into nearest points on new grid
;   then pick lstart and lcnt from this and lay out new grid

lstart=float(fix(l_0)) + float(fix((l_0-fix(l_0))*lf))/float(lf)
lstop =float(fix(l_1)) + float(fix((l_1-fix(l_1))*lf))/float(lf) + l_del
lcnt=fix((lstop-lstart)/l_del)+1
l_hi  = lstart+findgen(lcnt)*l_del

; step THREE - use gridlo to create output data

newsig=5.0*l_del
finaldata = gridlo(sortdata,l_hi,sigma=newsig)

; variables not passed on:  sz, len, l_0, l_1, l_del, newsig
end

