PRO segmentmad,rawdata,count,l_lo,griddata,continuum,moddata,NDEGREE=ndegree

; version 3.0
;
; 17 Jul 02 - use [] for array subscripts for griddata (v3.0) 
;  7 Aug 01 - to print coefficients to log file (v2.0)
; 23 Apr 01 - converted from segmentmod.pro
;
; corrects spectra from individual detectors to fit a median spectrum
;   works in lo-res wavelength grid
; input:    rawdata - original raw data
;           count - array giving length of each vector in rawdata
;           l_lo - low-res wavelength grid
;           griddata - spike-rejected regridded data from segmentlow.pro
;           continuum - the median of griddata from segmentlow.pro
; options:  ndegree - degree of quadratic (1=line, 2=quadratic, 3=cubic)
; output:   moddata - rawdata after corrections

; preliminaries, check input parameters and set defaults if necessary

if (n_elements(ndegree) eq 0) then ndegree = 2
ndegree=fix(ndegree)
if (ndegree gt 3) then begin
  print,'Error.  ndegree gt 3.  Cannot fit more than a cubic'
  stop
endif

istop=max(size(count)) ; OK since count is one dimensional
len=max(size(l_lo)) ; OK since l_lo is one dimensional

; step ONE - fit function to error(lambda) for each detector
;   error = regridded data/continuum
;   error must be double so poly_fit gives good results for cubic
;   fitted function can be linear, quadratic, or cubic
; commands to print coefficients, plot results are intermixed (commented out)

error=dblarr(len)
coeff=dblarr(istop,4)
coeff(*,*)=0.0

printf,99,'  segmentmad (additive) coefficients for each row: '
for i=0,istop-1 do begin
  error=double(griddata[i,*]-continuum)
  coeff_col=poly_fit(l_lo,error,ndegree,fit_data)
  printf,99,format='("    ",i2,"  ",4(f12.4))',i,coeff_col(0:ndegree)
  coeff(i,0:ndegree)=coeff_col
endfor
;!p.multi=0

; step TWO - apply corrections to raw data
moddata=rawdata
for i=0,istop-1 do begin
  for j=0,count(i)-1 do begin
    l=double(rawdata(0,i,j))
    correction=coeff(i,0)+coeff(i,1)*l
    if (ndegree gt 1) then correction = correction + coeff(i,2)*l*l 
    if (ndegree gt 2) then correction = correction + coeff(i,3)*l*l*l
    moddata(1,i,j)=rawdata(1,i,j)-correction
  endfor
endfor

; variables not passed on:  istop, len, error, coeff, i, coeff_col, 
;   j, l, correction

end
