PRO segmentmed,rawdata,count,lam,griddata,continuum,flag,NPASS=npass,RANGE=range,THRESHOLD=threshold

; version 4.0
; 17 Jul 02  use [] for array subscripts for griddata (v 4.0)
; 27 Feb 02  stop forcing all median data to be positive (v 3.0)
;            modify flag criteria to trigger if min < 15 or max < 20
; 23 Apr 01  add flag variable, set when median data < 0
; 29 Dec 00  remove all non-positive values in continuum
;            renamed from segmentlow to segmentmed
;  4 Oct 00  first modification
;
; finds a median continuum spectrum for one spectral segment
;  step ONE - remove spikes (and narrow features) from data
;  step TWO - regrid data to input wavelength grid
;  step THREE - combine data using a median filter
;
; input:    rawdata array
;           count - count array, gives actual length of vectors in rawdata
;           lam - input wavelength grid 
; options:  npass - number of calls to spike.pro, the spike rejection filter
;           range - passed to spike.pro, no. of pixels to each side to consider
;           threshold - passed to spike.pro, max sigma for data rejection
; output:   griddata - data regridded to input lambda grid
;           continuum - the final cleaned and medianed result
;                       passed back as function result

; preliminaries - check input parameters and set defaults if necessary
;  in spike.pro, defaults are different than here
;     range default = 2 here, but = 1 in spike.pro
;     threshold default = 1.5 here, but = 3.0 in spike.pro

if (n_elements(npass) eq 0) then npass=2
npass=fix(npass)
if (n_elements(range) eq 0) then range=1
range=fix(range)
if (n_elements(threshold) eq 0) then threshold=1.5

istop=max(size(count)) ; OK since count is 1-D
len=max(size(lam))     ; OK since wavelength grid is 1-D
flag=0                 ; no negative data in median (yet)

; step ONE - spike rejection filter, calls to spike.pro
; should be stringent to remove all narrow features, whether spikes or real
; in this step, rawdata is now cleaned, no longer really raw

tempdata=rawdata
for n=1,npass do begin
  for i=0,istop-1 do begin
    tempdata(1,i,0:count(i)-1) = $
      spike(tempdata(1,i,0:count(i)-1),range=range,threshold=threshold)
  endfor
endfor

; step TWO -  regrid all spectra onto input wavelength grid (lam)
; make call to gridlo.pro for actual regridding
; have to use tmp_old and tmp_new to fit needed dimensions for input and
;   output to gridlo.pro

griddata=fltarr(istop,len)
for i=0,istop-1 do begin
  tmp_old=fltarr(2,count(i))
  tmp_old(*,*)=tempdata(*,i,0:count(i)-1)
  tmp_new=gridlo(tmp_old,lam)
  griddata[i,*]=tmp_new
endfor

; step THREE - generate continuum
;   median all spectra into ONE
;   set flag if min of data < 15 Jy or max < 20 Jy
;   no longer setting non-positive or zero values to minimum positive value

continuum=lam
for i=0,len-1 do continuum(i)=median(griddata[*,i])

if (min(continuum) lt 15 or max(continuum) lt 20) then flag=1

; variables not passed on:  istop,len,n,i,tempdata,tmp_old,tmp_new

end
