PRO segmentmrg,median_n,median_p,flag_n,flag_p,wt_n,median,flag,MPASS=mpass

; version 4.1
; 17 Apr 02 - correct bug that leads to crash if flagspot=-1
; 29 Mar 02 - corrected bug that limited some flags (v 4.0)
;  6 Aug 01 - to print warning message to log file (v 2.0)
;  3 Aug 01 - created
;
; merges medians from down and up scan into one median
;   if both flags=0 or if wt_n=0 or 1
;     then makes simple weighted average, same weight at all wavelengths
;   if one flag=1, then checks more carefully
;     where data diverge from each other by more than mpass (typically 20%)
;     determines which scan direction is better and re-weights accordingly
;     means that weight can change with wavelength
;
; input:    median_n and median_p - medians for 2 scan directions
;           flag_n and flag_p - flags for 2 scan directions
;           wt_n - weight of down scan (wt_p = 1.0 - wt_n)
; options:  mpass - fractional amount by which two scans can differ from avg
;                   default = 0.20
; output:   median - combined  median
;           flag - combined flag

; preliminaries, check mpass, set wt_p, set flag

if (n_elements(mpass) eq 0) then mpass=0.2
mpass=float(mpass)

wt_p = 1.0 - wt_n

flag=0
if ( (flag_n eq 1) or (flag_p eq 1) ) then flag=1

; perform series of checks to see if we can use simple weighted avg
; if flag not set or only 1 scan direction used, then simple=1
; simple=1 means we will use simple weighted average
;   can skip most if blocks

if ( (flag eq 0) or (wt_n eq 0.0) or (wt_n eq 1.0) ) then $
  simple=1 else simple=0

if (simple eq 0) then begin

; must now compare scan directions for differences larger than mpass
; locations flagged in dif_flag
; if only 2 or less, then set simple=1

  istop=max(size(median_n)) ; OK since median_n is 1-D
  dif_flag=intarr(istop)
  dif_flag(*)=0

  diff =  median_n-median_p
  avg  = (median_n+median_p)/2.0
  flagspot = where (abs(diff/avg) gt mpass)
  if (total(flagspot) gt 0) then dif_flag(flagspot) = 1

  if (total(dif_flag) le 2) then simple=1

endif

if (simple eq 0) then begin

; simple still not 1, must look at distribution of flags in dif_flag
; any that are less than 3 in a row set these to 0
; end by checking again if we can set simple=0

  cnt=intarr(istop)

  for i=0,istop-1 do begin
    if (dif_flag(i) eq 1) then begin
      count=0
      j=i
      while (j lt istop) do begin
        if (dif_flag(j) eq 1) then count=count+1 else break
        j=j+1
      endwhile
      cnt(i:j-1)=count
      i=j
    endif
  endfor

  for i=0,istop-1 do begin
    if (cnt(i) lt 3) then begin
      cnt(i)=0
      dif_flag(i)=0
    endif
  endfor

  if (max(cnt) le 2) then simple=1
endif

if (simple eq 0) then begin

;  since simple still not 1, must modify weights
;    first, find mean of non-controversial pixels in 2 scan directions
;    then set weight to take whichever one is closer to mean
;    must do this in groups

  print,'segmentmrg is using variable weights to combine medians'
  printf,99,'  segmentmrg is using variable weights to combine medians'

; create weight vectors

  weight_n=fltarr(istop)
  weight_n(*)=wt_n

; find averages

  avg_n=0.0
  avg_p=0.0

  for i=0,istop-1 do if (dif_flag(i) eq 0) then begin
    avg_n = avg_n + median_n(i)
    avg_p = avg_p + median_p(i)
  endif
  avg_n = avg_n / (istop-total(dif_flag))
  avg_p = avg_p / (istop-total(dif_flag))

; make appropriate replacements in weight vectors
;   this loop is designed to replace more than one wavelength block if needed

  i=0
  while (i lt istop-1) do begin
    j=i
    while ( (j lt istop-2) and (dif_flag(j) eq 0) ) do j=j+1
    if (j lt istop-2) then begin
      test_n=total(median_n(j:j+cnt(j)-1))/float(cnt(j))
      test_p=total(median_p(j:j+cnt(j)-1))/float(cnt(j))
      if (abs(test_n-avg_n) gt abs(test_p-avg_p)) then $
        weight_n(j:j+cnt(j)-1)=0.0 else weight_n(j:j+cnt(j)-1)=1.0
      i=j+cnt(j)
    endif else i=istop
  endwhile
  weight_p = 1.0-weight_n      ; reminder - these are vectors

; calculate median using weight vectors

  median = weight_n*median_n + weight_p*median_p

endif else begin 

; simple=1 so calculate median using simple weighted average

  median = wt_n*median_n + wt_p*median_p / (wt_n+wt_p)

endelse

end
