PRO segmentpass,moddata,count,l_lo,continuum,pass

; version 4.0
; 25 Apr 02 - change step THREE to use standard deviation
; 23 Jul 01 - comment changes only
; 14 Dec 00 - minor modifications only
;  4 Oct 00
;
; generates an array of typical deviations from the median spectrum
;   will be used when merging data from individual detectors to
;   reject spikes and bad data
;
; input:    moddata - data on original wavelength grid
;                     corrected for curvature, but not spike-rejected
;           count - array giving length of each vector in rawdata
;           l_lo - low-res wavelength grid
;           continuum - the median of griddata from segmentlow.pro
; output:   pass - single array of passable differences on low-res grid

; step ONE - shift moddata to low-res wavelength grid
; store result in gmdata

istop=max(size(count)) ; OK since count is 1-D
len=max(size(l_lo))    ; OK since l_lo is 1-D
gmdata=fltarr(istop,len)

for i=0,istop-1 do begin
  tmp_old=fltarr(2,count(i))
  tmp_old(*,*)=moddata(*,i,0:count(i)-1)
  tmp_new=gridlo(tmp_old,l_lo)
  gmdata(i,*)=tmp_new
endfor

; step TWO - find difference between continuum and gmdata

diff=fltarr(istop,len)
for i=0,istop-1 do diff(i,*)=gmdata(i,*)-continuum

; step THREE - find allowable difference at each wavelength
; store result in pass array
; at each wavelength, take larger of:
;   medabs = median of absolute value
;   stddev = standard deviation

pass=l_lo 
for j=0,len-1 do begin
  medabs=median(abs(diff(*,j)))
  stddev=sqrt(total(diff(*,j)*diff(*,j))/float(istop-1))
  pass(j)=stddev
  if (medabs gt stddev) then pass(j) = medabs
endfor
; variables not passed on - istop, len, gmdata, i, tmp_old, tmp_new, diff,
;  medabs, stddev

end
