FUNCTION segmentread,file,rawdata,count,l_lo

; version 4.1
; 20 Jan 03  modify to skip detectors with 1 or fewer pixels
; 26 Apr 02  rewrote as a function, modify for array lengths of 0
; 27 Feb 02  cosmetic changes only
;  8 Oct 01  to change nskip fr 6 to 10
;            this sets how big of a data gap to skip over
; 29 Dec 00  new algorithm to set lo-res wavelength grid
;            to avoid gaps at end of some segments
; 14 Dec 00  correct for detectors with zero counts
;  4 Oct 00
;
; first step in algorithm to produce one spectrum from 12 detectors
;   input:   file name
;   output:  rawdata (2,12,len) (len = max elements in any detector)
;            count (12) - length of all 12 vectors in rawdata
;            l_lo (len) - low-resolution lambda array
;

nskip=10.0 ; parameter, was 6 in version 2.0

; step ONE - load data into 3-column format (listdata)
;            and load count

detcount=intarr(49)
close,1
openr,1,file
readf,1,detcount
if (total(detcount(1:48)) le 0) then begin
  status=0
  return,status
endif else begin
  status=1
endelse
listdata=fltarr(3,total(detcount(1:48)))
readf,1,listdata
close,1

; load count array (these lines rewritten 14 Dec 00)
;   check for detectors with zero pixels
;   reduce size of count to compensate and skip these data

ndet=0
for i=0,48 do if (detcount(i) gt 1) then ndet = ndet+1
count  = intarr(ndet)
cstart = count
cstop  = count
for i=0,3 do begin ; load count with non-zero block of 12 integers
  istart=1+(i*12)
  if (total(detcount(istart:istart+11)) gt 0) then begin
    jspot=0
    for j=0,11 do begin
      if (detcount(istart+j) gt 1) then begin
        count(jspot)  = detcount(istart+j)
        cstart(jspot) = total(detcount(0:istart+j-1))
        cstop(jspot)  = cstart(jspot)+count(jspot)-1
        jspot = jspot+1
      endif
    endfor
  endif
endfor

; step TWO - separate spectra from all good detectors into rawdata array
;            note - number (ndet) might be less than 12 now

rawdata=fltarr(2,ndet,max(count))
for i=0,ndet-1 do begin
  rawdata(0:1,i,0:count(i)-1)=listdata(0:1,cstart(i):cstop(i))
endfor

; step THREE - create low resolution wavelength grid
;              will run from lmin to lmax 
;              using smallest median increment of the 12 detectors
;              for each detector, find ldeltest, lmintest, lmaxtest
;              then save min ldel, min lmin, max lmax

lmin=0.0
lmax=0.0
ldel=0.0

for i=0,ndet-1 do begin
  delta=fltarr(count(i)-1)
  for j=0,count(i)-2 do delta(j)=rawdata(0,i,j+1)-rawdata(0,i,j)
  ldeltest=median(delta)
  if (count(i) gt 2) then begin
    j0=count(i)/2
    while ((j0 gt 0) and $ ; crash here
      (rawdata(0,i,j0+1)-rawdata(0,i,j0) lt nskip*ldeltest)) do j0=j0-1
    lmintest=rawdata(0,i,j0)
    j1=count(i)/2
    while ((j1 lt count(i)-2) and  $
      (rawdata(0,i,j1+1)-rawdata(0,i,j1) lt nskip*ldeltest)) do j1=j1+1
    lmaxtest=rawdata(0,i,j1)
  endif else begin
    lmintest=rawdata(0,i,0)
    lmaxtest=rawdata(0,i,count(i)-1)
  endelse
  if (i eq 0 or ldeltest lt ldel) then ldel=ldeltest
  if (i eq 0 or lmintest lt lmin) then lmin=lmintest
  if (i eq 0 or lmaxtest gt lmax) then lmax=lmaxtest
endfor
len=fix((lmax-lmin)/ldel)+1
l_lo=lmin+findgen(len)*ldel

; variables not passed on:  detcount, listdata, ndet, i, istart, istop
;   jspot, delta, ldel, ldeltest, lmintest, lmaxtest, lmin, lmax, len

return,status
end
