PRO segmentsort,moddata,count,l_lo,continuum,pass,sortdata

; version 2.0
; last modified 3 Oct 00
;
; sorts data from all detectors into one spectrum, dropping data
;   which are further from the continuum than allowed by the array pass
; input:   moddata - data on original wavelength grid
;                    corrected for curvature, but not spike-rejected
;          count - array giving length of each vector in rawdata
;          l_lo - low-res wavelength grid
;          continuum - the median of griddata from segmentlow.pro
;          pass - array of maximum acceptable differences from continuum
; output:  sortdata - wavelength and flux for sorted good data

; step ONE - flag bad pixels
; for each detector, must
;   - shift pass and continuum to its wavelength grid
;   - compare actual diff betw moddata and continuum to pass

istop=max(size(count)) ; OK since count is 1-D
len=max(size(l_lo)) ; OK since count is 1-D
flag=intarr(istop,max(count))
flag(*,*)=0 ; set all flags to be bad initially
cont_old=fltarr(2,len)
pass_old=fltarr(2,len)
cont_old(0,*)=l_lo
pass_old(0,*)=l_lo
cont_old(1,*)=continuum
pass_old(1,*)=pass
for i=0,istop-1 do begin
  new_lam=fltarr(count(i))
  new_lam(*)=moddata(0,i,0:count(i)-1)
  cont_new=gridlo(cont_old,new_lam)
  pass_new=gridlo(pass_old,new_lam)
  diff=fltarr(count(i))
  diff=abs(moddata(1,i,0:count(i)-1)-cont_new)
  for j=0,count(i)-1 do begin
    if (diff(j) le pass_new(j)) then flag(i,j)=1 ; set flag = good
  endfor
endfor

; step TWO - put moddata(0,*,*), moddata(1,*,*), and flag into 1-D arrays

big_len=total(count)
signal=fltarr(big_len)
lambda=fltarr(big_len)
flagvector=intarr(big_len)
start=0
for i=0,istop-1 do begin
  last_det=count(i)-1
  stop=start+last_det
  lambda(start:stop) = moddata(0,i,0:last_det)
  signal(start:stop) = moddata(1,i,0:last_det)
  flagvector(start:stop) = flag(i,0:last_det)
  start=start+last_det+1
endfor

; step THREE - use IDL sort algorithm to organize data by wavelength
; load into sortdata (col 1 = wavelength, col 2 = flux, col 3 = flag)

tempdata=fltarr(3,big_len)
spot=sort(lambda)
tempdata(0,*)=lambda(spot)
tempdata(1,*)=signal(spot)
tempdata(2,*)=flagvector(spot)

; step FOUR - copy into final array, skipping data for which flag = 0
; final array has only wavelength and flux columns

sortdata=fltarr(2,total(flag))
i=0 & j=0
while (j lt big_len) do begin
  if (tempdata(2,j) eq 1) then begin
    sortdata(0:1,i) = tempdata(0:1,j)
    i=i+1
  endif
  j=j+1
endwhile

; variables not passed on:  len, flag, cont_old, pass_old, i, new_lam, 
;   cont_new, pass_new, diff, l_hi, big_len, start, signal, lambda, 
;   flagvector, last_det, stop, spot, tempdata, j

end
