FUNCTION spike,data_array,RANGE=range,THRESHOLD=threshold

; version 2.1
; 30 Apr 02 to handle arrays with no good data
; 19 Mar 02 to eliminate crashes when width < size of data_array
; 27 Sep 00 trivial changes
; 18 Jun 00 major revisions
;  7 Oct 97
;
; a spike rejection filter for 1-D data
; rejects N-sigma variations (with the 2M neighboring pixels)
; sigma array smoothed by median filtering
; replaces bad pixels with avg of neighbors

dat=data_array
q=size(dat)
istop=q(q(0)+2)-1

if (istop lt 2) then return,dat ; array too short, returning original data
sig=dblarr(istop+1)
flag=intarr(istop+1)
flag(*)=1
if (n_elements(range) eq 0) then range=1
range=fix(range)
if (n_elements(threshold) eq 0) then threshold=3.0
if (range lt 1) then begin
  print,'Error.  Range must be one or more.'
  stop
endif
;
; first pass:  create sigma array
;
for i=0,istop do begin
  jstart=i-range
  jstop=i+range
  if (jstart lt 0) then begin
    jstart=0
    if (jstop lt 2) then jstop=2
  endif
  if (jstop gt istop) then begin
    jstop=istop
    if (jstart gt istop-2) then jstart=istop-2
  endif
  count=float(jstop-jstart) ; guaranteed to be at least 2
  sum=0.0
  sumsq=0.0
  for j=jstart,jstop do begin
    if (j ne i) then begin
      sum = sum + double(dat(j))
      sumsq = sumsq + double(dat(j))*double(dat(j))
    endif
  endfor
  sig(i)=sqrt((sumsq - sum*sum/count)/(count-1.0))
endfor
;
; second pass:  median sigma array
;   median width must include more than twice the pixels
;   covered in first pass
;
width=4*range+3
if (width gt n_elements(sig)) then width=n_elements(sig) ; added 19 Mar 02
sig=median(sig,width)
;
; third pass:  flag bad pixels using a median filter
;
for i=0,istop do begin
  jstart=i-range
  if (jstart lt 0) then jstart=0
  jstop=i+range
  if (jstop gt istop) then jstop=istop
  avg=median(dat(jstart:jstop))
  if (abs(dat(i)-avg) ge threshold*sig(i)) then flag(i)=0
endfor
;
; fourth pass:  replace bad pixels by interpolating between 
;   nearest good neighbors
;   must find first good pixel at each end of array
;
i=0
while (flag(i) eq 0 and i lt istop-1) do i=i+1
dat(0:i)=dat(i+1)
istart=i
i=istop
while (flag(i) eq 0 and i gt 1) do i=i-1
dat(i:istop)=dat(i-1)
istop=i
for i=istart,istop do begin
  if (flag(i) eq 0) then begin
    jstart=i
    while (flag(jstart) eq 0 and jstart gt 1) do jstart=jstart-1
    jstop=i
    while (flag(jstop) eq 0 and jstop lt istop-1) do jstop=jstop+1
    if (jstop ne jstart) then begin
      dat(i)=dat(jstart)+(dat(jstop)-dat(jstart))*(i-jstart)/(jstop-jstart)
    endif else begin
      dat(i)=0.0
    endelse
  endif
endfor

return,dat
end
