FUNCTION splice_two,la,fa,la0,la1,lb,fb,lb0,lb1,lc,fc,lc0,lc1,MODE=mode

; version 4.3
; 23 Jul 02 - to eliminate varoius crashes due to bad data
; 21 Jul 02 - created
;
; normalizes a segment to the average of two adjacent segments over specified wavelengths
; finds multiplicative or additive factor to splice two spectral segmennts
; fits second to first and third
; uses segmentavg to find average flux for wavelength range of each segment
; determines multiplicative or additive correction
; input:
;	la,lb,lc = spectral wavelengths
;	fa,fb,fc = spectral fluxes (must be same length as la,lb)
;	la0,lb0,lc0 = starting wavelengths
;	la1,lb1,lc1 = stopping wavelengths
;       mode   = 0 for multiplicative correction
;              = 1 for additive correction
; output:  array of two factors
;        first is multiplicative or additive correction
;        second is error in correction, always in flux units

; define the polynomials to be used

if (la0 gt la1) then begin & temp=la0 & la0=la1 & la1=temp & endif
if (lb0 gt lb1) then begin & temp=lb0 & lb0=lb1 & lb1=temp & endif
if (lc0 gt lc1) then begin & temp=lc0 & lc0=lc1 & lc1=temp & endif

; check to see if segments overlap wavelength ranges and set stops

error=0
if (min(la) gt la1 or max(la) lt la0) then error=1
if (min(lb) gt lb1 or max(lb) lt lb0) then error=1
if (min(lc) gt lc1 or max(lc) lt lc0) then error=1

if (error eq 0) then begin

; call segmentavg for three segments, find avg of neighbors and uncertainty

  value_a = segmentavg(la,fa,la0,la1)
  value_b = segmentavg(lb,fb,lb0,lb1)
  value_c = segmentavg(lc,fc,lc0,lc1)

  value_d = (value_a+value_c)/2.0
  sig_d   = sqrt( ((value_a^2 - value_d^2) + (value_c^2 - value_d^2))/2.0 )
  sig_b   = 0.0

endif else begin

  print,format=$
    '("unable to perform splice_two centered at ",f6.3)',0.5*(lb0+lb1)
  print,format=$
    '("  data range ",f6.3,"-",f6.3,", splice range ",f6.3,"-",f6.3)',$
    min(la),max(la),la0,la1
  print,format=$
    '("  data range ",f6.3,"-",f6.3,", splice range ",f6.3,"-",f6.3)',$
    min(lb),max(lb),lb0,lb1
  print,format=$
    '("  data range ",f6.3,"-",f6.3,", splice range ",f6.3,"-",f6.3)',$
    min(lc),max(lc),lc0,lc1
  printf,99,format=$
    '("  unable to perform splice_out centered at ",f6.3)',0.5*(lb0+lb1)
  printf,99,format=$
    '("    data range ",f6.3,"-",f6.3,", splice range ",f6.3,"-",f6.3)',$
    min(la),max(la),la0,la1
  printf,99,format=$
    '("    data range ",f6.3,"-",f6.3,", splice range ",f6.3,"-",f6.3)',$
    min(lb),max(lb),lb0,lb1
  printf,99,format=$
    '("    data range ",f6.3,"-",f6.3,", splice range ",f6.3,"-",f6.3)',$
    min(lc),max(lc),lc0,lc1

endelse

; print,value_a,value_c,value_d,sig_d

c=fltarr(2) & c(*)=1.0
if (mode ne 1) then begin
    if (error eq 0) then begin 
        c(0) = value_d/value_b
        c(1) = sqrt( (sig_d/value_b)^2 + (sig_b*value_d/value_b^2)^2 )
    endif else begin
        c(*)=1.0
    endelse
endif else begin
    if (error eq 0) then begin
        c(0)= value_d-value_b   
        c(1)= sqrt(sig_d^2 + sig_b^2)
    endif else begin 
        c(*)=0.0
    endelse
endelse

return,c
end
