pro swsmake2,TYPE=type

; version 4.4
;  6 Sep 02  to use Band 2C out to 12.40 um, splice 12.35-12.55 um (v 4.4)
; 22 Jul 02  to call new seg2segtwo to normalize Band 3E (v 4.3)
;  8 May 02  to include type variable in log file
;  5 Apr 02  propagation of errors now handled by seg2segin
; 29 Mar 02  to print normalizations and errors to log file, not data file
;            modify output format, modify normalization wavelengths for 2A/2B
;            modify cut ranges to prevent two data at same wavelength
; 19 Mar 02  modify calls to cut.pro to pass len - length of cut arrays
; 14 Mar 02  call seg2segout.pro to handle call to splice_out.pro
;            also propagate errors as with seg2segin.pro
;  7 Mar 02  propagates segment-to-segment errors to seg2segin.pro
;  1 Mar 02  allows new types 10-13, like types 0-3 but for em line sources
; 27 Feb 02  now calls seg2segin.pro, which calls splice_in.pro
;            reformatting to add one character to each entry in header rows
;            reduce columns for flux and errors, add another error column
; 10 Sep 01  to start with Band 1E and make additive correction for low fluxes
;  9 Aug 01  to handle errors from cut.pro
;  8 Aug 01  to modify format of header lines in all.sws file
;  7 Aug 01  to write to log file
; 31 Jul 01  to read/write 3rd header line - normalizations
; 27 Jul 01  to debug
; 25 Jul 01  for new splice and cut wavelengths
; 24 Jul 01  to extensively modify swstestf.pro
;  5 Jun 01  to change cut lamdas
; 23 Apr 01  to shift wavelength range for 3D/4 extrapolation
; 20 Apr 01  to use quadratic for Band 3D/4 extrapolation
; 27 Feb 01  first modification
;
; the second half of the SWS make procedure
;
; this is the procedure to load and normalize spectral segments
; segment-to-segment normalization optimized for sirius
;   should work for other spectra just fine
; wavelengths used in calls to splice and cut are a first 
;   approximation only - they need to be tested more carefully
;
; input parameter - TYPE
;   type =  0   standard quadratic normalization
;           1   modify Band 3D/4 normalization for 26 um feature
;           2   modify Band 3D/4 normalization for some red sources
;           3   modify Band 3D/4 normalization for noisy red sources
;   type = 10-3 as type 0-3, except calls seg2segin with 4 wavelengths, not 2
;               so seg2segin will call splice_in2 instead of splice_in
;               for emission line sources
;
; prints out result in file all.sws

; read in file all.pws 

len=intarr(12)
flag=intarr(12)
nfact=fltarr(12)
nferr=fltarr(12)
close,1
openr,1,'all.pws'
readf,1,len
readf,1,flag
; readf,1,nfact

data=dblarr(4,12,max(len))
for i=0,11 do begin
  dum=fltarr(4,len(i))
  readf,1,dum
  data(*,i,0:len(i)-1)=dum
endfor
close,1

; open log file "all.log"
; check TYPE, print out to log file before modifying
; then print more to log file

close,99
openw,99,'all.log',/append
printf,99,''
printf,99,'Log information from swsmake2.pro'

em_flag=0
if (n_elements(type) eq 0) then type=0  ; set to default if undeclared
printf,99,'' 
printf,99,format='("Splicing information (type = ",i2,"):")',type
printf,99,''
if (type ge 10) then begin              ; set em_flag and reset type
  em_flag=1
  type = type-10 
endif
if (type gt 3 or type lt 0) then type=0 ; set to default if out of range


; check flux levels, if max < 20 Jy or min < 15 Jy for a segment, flag=1
; this step is now redundant with modifications to segmentmed.pro

for i=0,11 do begin
  maxdata=max(data(1,i,0:len(i)-1))
  mindata=min(data(1,i,0:len(i)-1))
  if (maxdata lt 20 or mindata lt 15) then flag(i)=1
endfor

; reset normalization factors for each segment
; make calls to seg2segin.pro and seg2segout.pro, 
;  which call the splice routines
; flag=0, multiplicative, flag=1, additive

for i=0,11 do if (flag(i) eq 0) then nfact(i)=1.0 else nfact(i)=0.0

;  i  Band  Segment  Splicing
;  0  1A     1       1-2
;  1  1B     2       2-3
;  2  1D     3       3-4
;  3  1E     4       4-5
;  4  2A     5       5-6
;  5  2B     6       6-7
;  6  2C     7       7-9
;  7  3A     9       9-10
;  8  3C    10       10-11
;  9  3D    11
; 10  3E    12       
; 11  4     13

seg=[1,2,3,4,5,6,7,9,10,11,12,13]

; splicing ranges

la=[2.562,3.000,3.351,4.019,5.265,7.02,12.25,16.00,19.37]  ; 11.99
lb=[2.630,3.043,3.670,4.082,5.400,7.06,12.55,16.56,19.56]  ; 12.56
lia=[2.596,3.022,3.511,4.047,5.473,7.04,12.40,16.28,19.47] ; 12.28
lib=[2.596,3.022,3.511,4.057,5.473,7.04,12.40,16.28,19.47] ; 12.28
;la=[2.56,2.99,3.24,4.01,5.40,6.95,12.40,16.18,19.35]  ; stopped using
;lb=[2.60,3.03,3.75,4.06,5.50,7.00,12.57,16.38,19.42]  ;   on 25 Jul 01
;lb=[2.630,3.043,3.670,4.082,5.680,7.06,12.56,16.56,19.56] ; replaced 27 Mar 02

; find and apply normalizations for segments 5-11
; direction:  start from brighter of the two, go to the fainter
; use splice_in in all cases

i=3 ; for Band 1E = segment 4
ff05=segmentavg(data(0,i,0:len(i)-1),data(1,i,0:len(i)-1),0.0,0.0)

i=9; for Band 3D = segment 11
ff11=segmentavg(data(0,i,0:len(i)-1),data(1,i,0:len(i)-1),0.0,0.0)

if (ff05 ge ff11) then begin ; splice fr Band 1E (4.05 um) to Band 3D
  istart=4 & istop=9
  id=1     & il=-1
  print,'normalization begins at Band 1E (Seg. 4)' 
endif else begin ; splice from Band 3D (19.39 um) to Band 1E
  istart=8 & istop=3
  id=-1    & il=0
  print,'normalization begins at Band 3D (Seg. 11)'
endelse

prop_err = 0.0 ; zero propagated segment-to-segment error

for i=istart,istop,id do begin
  dum=data(*,i,0:len(i)-1)
  if (em_flag eq 0) then begin
    seg2segin,data(*,i-id,0:len(i-id)-1),seg(i-id),dum,seg(i),$
      [la(i+il),lb(i+il)],flag(i),nfa
  endif else begin
    seg2segin,data(*,i-id,0:len(i-id)-1),seg(i-id),dum,seg(i),$
      [la(i+il),lia(i+il),lib(i+il),lb(i+il)],flag(i),nfa
  endelse
  nfact(i)=nfa(0)
  nferr(i)=nfa(1)
  data(*,i,0:len(i)-1)=dum
endfor

; find splicing constants for segments 3-1 using splice_in in all cases

istart=2 & istop=0
id=-1    & il=0
  
; zero propagated error if we started at band 1E

if (ff05 ge ff11) then begin
  old_err = prop_err         ; saving this for following splice_out calls
  prop_err = 0.0 
endif

for i=istart,istop,id do begin
  dum=data(*,i,0:len(i)-1)
  if (em_flag eq 0) then begin
    seg2segin,data(*,i-id,0:len(i-id)-1),seg(i-id),dum,seg(i),$
      [la(i+il),lb(i+il)],flag(i),nfa
  endif else begin
    seg2segin,data(*,i-id,0:len(i-id)-1),seg(i-id),dum,seg(i),$
      [la(i+il),lia(i+il),lib(i+il),lb(i+il)],flag(i),nfa
  endelse
  nfact(i)=nfa(0)
  nferr(i)=nfa(1)
  data(*,i,0:len(i)-1)=dum
endfor

; splice from Band 3D to 4, using extrapolation to avoid light leak
; use splice_out to get from Band 3D to Band 4
; have multiple choices for wavelength ranges depending on nature of source

; if we started at band 1E, zero propagated error 
; otherwise reload old propagated error

if (ff05 ge ff11) then prop_err=old_err else prop_err=0.0

i=11 & id=2
dum=data(*,i,0:len(i)-1)
seg2segout,data(*,i-id,0:len(i-id)-1),seg(i-id),dum,seg(i),$
  type,flag(i),prop_err,nfa
nfact(i)=nfa(0)
nferr(i)=nfa(1)
data(*,i,0:len(i)-1)=dum

; splice from Band 4 to Band 3E using splice_out - special case

i=10 & id=1
dum=data(*,i,0:len(i)-1)
seg2segtwo,data(*,i-id,0:len(i-id)-1),seg(i-id),dum,seg(i),$
  data(*,i+id,0:len(i+id)-1),seg(i+id),flag(i),prop_err,nfa
nfact(i)=nfa(0)
nferr(i)=nfa(1)
data(*,i,0:len(i)-1)=dum

; print review of normalization and start of cutting to log file

printf,99,''
printf,99,'Segment  length  flag     norm.      error'
for i=0,11 do printf,99,format='(i7,i8,i6,f11.3,f11.3)',$
  seg(i),len(i),flag(i),nfact(i),nferr(i)
printf,99,''
printf,99,'Cutting information:'
printf,99,''

; make calls to cut.pro to truncate bad data from ends of each segment
; new wavelengths based partially on official cut-offs and my own
; eliminate overlaps

; new cut ranges to avoid adjacent segments with data at same lambda
l0=[ 2.35   , 2.60008, 3.02008, 3.52013, 4.08013,5.30025,7.00025,$
    12.40042,16.50042,19.50083,27.30083,27.70125] ; 12.00042
l1=[l0(1:11),45.39]
; cut ranges used fr 25 Jul 01 to 28 Mar 02, see notes 25 Jul 01 for details
;l0=[2.35,2.60,3.02,3.52,4.08,5.30,7.00,12.0,16.5,19.5,27.3,27.7]
; cut ranges used from 5 Jun to 25 Jul 01, overlaps eliminated
;l0=[2.35,2.60,3.02,3.52,4.06,5.40,7.00, 12.40,16.38,19.42,27.3,27.7]
; the original cuts - no longer used since they leave overlaps
;l0=[2.35,2.56,2.99,3.24,4.01,5.40,6.95, 12.40,16.18,19.35,27.30,28.9]
;l1=[2.60,3.03,3.75,4.06,5.50,7.05,12.57,16.38,19.42,27.30,29.0,45.39]

for i=0,11 do begin
  cut,data(0,i,0:len(i)-1),data(1,i,0:len(i)-1),l0(i),l1(i),l,f,lendum
  cut,data(0,i,0:len(i)-1),data(2,i,0:len(i)-1),l0(i),l1(i),l,es
  cut,data(0,i,0:len(i)-1),data(3,i,0:len(i)-1),l0(i),l1(i),l,ep
  data(*,i,*)=0.0
  len(i)=lendum
  if (len(i) gt 0) then begin
    data(0,i,0:len(i)-1)=l
    data(1,i,0:len(i)-1)=f
    data(2,i,0:len(i)-1)=es
    data(3,i,0:len(i)-1)=ep
    printf,99,format= '("  segment ",i2," cut to ",f6.3,"-",f6.3," um")',$
      seg(i),l0(i),l1(i)
  endif else begin
    printf,99,format='("  segment ",i2," cut to no data")',seg(i)
  endelse
endfor

; print to output data file, to screen, and to log file

close,9
openw,9,'all.sws'
printf,9,format='(12(i6))',len
printf,9,format='(12(i6))',flag 

print,'Segment  length  flag     norm.      error'
for i=0,11 do print,format='(i7,i8,i6,f11.3,f11.3)',$
  seg(i),len(i),flag(i),nfact(i),nferr(i)

printf,99,''
printf,99,format='("Seg.:",12(i6))',seg
printf,99,format='("Len.:",12(i6))',len

; print out data - avoid segments cut to zero length

for i=0,11 do begin
  if (len(i) gt 0) then for j=0,len(i)-1 do $
    printf,9,format='(f8.5,3(f9.2))',data(0:3,i,j)
endfor

close,9
close,99

end
